
# q-expansion of newform 275.3.c.e, downloaded from the LMFDB on 05 February 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 275
weight = 3
poly_data = [105, 0, 1, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, -1, 1, 0], 1], [[-10, 21, -3, 2], 41], [[-21, 40, 6, -4], 41]]

hecke_ring_character_values = [[177, [1, 0, 0, 0]], [101, [-1, 0, 0, 0]]]
aps_data = [[0, 0, -1, 0], [0, 0, 0, 1], [0, 0, 0, 0], [0, 0, -5, 0], [-4, 1, 0, 0], [0, 0, 4, 0], [0, 0, 7, 0], [0, -1, 0, 0], [0, 0, 0, 6], [0, 1, 0, 0], [-3, 0, 0, 0], [0, 0, 0, -1], [0, -2, 0, 0], [0, 0, 10, 0], [0, 0, 0, 14], [0, 0, 0, 1], [18, 0, 0, 0], [0, 7, 0, 0], [0, 0, 0, -6], [27, 0, 0, 0], [0, 0, 26, 0], [0, 6, 0, 0], [0, 0, -32, 0], [-37, 0, 0, 0], [0, 0, 0, -26], [0, 10, 0, 0], [0, 0, 0, -4], [0, 0, -70, 0], [0, 2, 0, 0], [0, 0, 0, 36], [0, 0, 14, 0], [0, 21, 0, 0], [0, 0, 0, 14], [0, -22, 0, 0], [0, 15, 0, 0], [0, -10, 0, 0], [0, 0, 0, -11], [0, 0, 0, -59], [0, 0, -133, 0], [0, 0, -64, 0], [-242, 0, 0, 0], [-258, 0, 0, 0], [-278, 0, 0, 0], [0, 0, -35, 0], [0, 0, 88, 0], [53, 0, 0, 0], [0, 17, 0, 0], [0, 0, 0, 86], [0, 0, 124, 0], [-292, 0, 0, 0], [0, 0, -77, 0], [0, -22, 0, 0], [0, -32, 0, 0], [282, 0, 0, 0], [0, 0, 0, 14], [0, 0, 89, 0], [-162, 0, 0, 0], [0, 24, 0, 0], [0, 0, -26, 0], [0, 16, 0, 0], [0, 0, -112, 0], [0, 0, -200, 0], [0, 0, -200, 0], [87, 0, 0, 0], [0, 0, 0, -64], [0, 0, 0, -11], [-138, 0, 0, 0], [0, 0, -109, 0], [0, 0, -2, 0], [0, -50, 0, 0], [0, 0, 0, -64], [0, 62, 0, 0], [0, 0, 0, 94], [0, 0, -244, 0], [428, 0, 0, 0], [0, 0, 0, 36], [228, 0, 0, 0], [0, 0, 0, -146], [107, 0, 0, 0], [0, 32, 0, 0], [-722, 0, 0, 0], [-408, 0, 0, 0], [0, 6, 0, 0], [0, 0, 0, -54], [0, -42, 0, 0], [0, 0, 0, 66], [-22, 0, 0, 0], [0, 0, -155, 0], [0, -33, 0, 0], [0, 0, 0, 136], [0, 0, 0, -121], [0, -4, 0, 0], [0, 0, 0, 74], [0, -57, 0, 0], [268, 0, 0, 0], [0, 0, 242, 0], [828, 0, 0, 0], [242, 0, 0, 0], [0, 0, -64, 0], [0, 43, 0, 0], [0, 0, 178, 0], [0, 0, -70, 0], [0, 0, 394, 0], [0, 14, 0, 0], [0, 59, 0, 0], [0, 0, 0, -36], [0, 0, 0, -191], [0, 0, -50, 0], [493, 0, 0, 0], [0, -10, 0, 0], [0, 0, 365, 0], [0, 0, 394, 0], [0, 0, 0, 64], [-972, 0, 0, 0], [-113, 0, 0, 0], [1007, 0, 0, 0], [0, 0, 0, -59], [0, 0, 0, 4], [0, 0, 0, 151], [0, 107, 0, 0], [862, 0, 0, 0], [0, 0, -319, 0], [0, 0, 14, 0], [0, 0, 0, 131], [872, 0, 0, 0], [0, -85, 0, 0], [768, 0, 0, 0], [263, 0, 0, 0], [0, 0, 0, 64], [0, 0, 98, 0], [0, 38, 0, 0], [0, 0, 305, 0], [-1003, 0, 0, 0], [0, 0, 0, -266], [0, -18, 0, 0], [0, 24, 0, 0], [0, 0, 0, 61], [0, 0, -334, 0], [0, 0, 0, 154], [0, -118, 0, 0], [0, 107, 0, 0], [0, 74, 0, 0], [0, 0, 0, -64], [0, 0, -266, 0], [-102, 0, 0, 0], [138, 0, 0, 0], [0, 0, -238, 0], [0, 0, 425, 0], [1068, 0, 0, 0], [0, 0, 0, 16], [0, 0, -76, 0], [-78, 0, 0, 0], [0, 0, 0, -339], [0, 0, -134, 0], [0, 0, 0, -151], [1037, 0, 0, 0], [0, 174, 0, 0], [633, 0, 0, 0], [0, 0, 326, 0], [0, 83, 0, 0], [0, 0, 0, 199], [0, 0, -553, 0], [0, 0, -193, 0], [-1048, 0, 0, 0], [0, 0, 0, 54], [0, 0, 0, 6], [1182, 0, 0, 0], [0, 0, 758, 0]]
